/*
 * tam3517.c  -- ALSA SoC support for OMAP3517 / AM3517 EVM
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any kind,
 * whether express or implied; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */

#include <linux/clk.h>
#include <linux/platform_device.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>

#include <asm/mach-types.h>
#include <mach/hardware.h>
#include <mach/gpio.h>
#include <plat/mcbsp.h>

#include "omap-mcbsp.h"
#include "omap-pcm.h"

#include "../codecs/tlv320aic23.h"

#define CODEC_CLOCK 	12000000

static int tam3517_hw_params(struct snd_pcm_substream *substream,
	struct snd_pcm_hw_params *params)
{
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct snd_soc_dai *codec_dai = rtd->dai->codec_dai;
	struct snd_soc_dai *cpu_dai = rtd->dai->cpu_dai;
	int ret;

	/* Set codec DAI configuration */
	ret = snd_soc_dai_set_fmt(codec_dai,
				  SND_SOC_DAIFMT_DSP_B |
				  SND_SOC_DAIFMT_NB_NF |
				  SND_SOC_DAIFMT_CBM_CFM);
	if (ret < 0) {
		printk(KERN_ERR "can't set codec DAI configuration\n");
		return ret;
	}

	/* Set cpu DAI configuration */
	ret = snd_soc_dai_set_fmt(cpu_dai,
				  SND_SOC_DAIFMT_DSP_B |
				  SND_SOC_DAIFMT_NB_NF |
				  SND_SOC_DAIFMT_CBM_CFM);
	if (ret < 0) {
		printk(KERN_ERR "can't set cpu DAI configuration\n");
		return ret;
	}

	/* Set the codec system clock for DAC and ADC */
	ret = snd_soc_dai_set_sysclk(codec_dai, 0,
			CODEC_CLOCK, SND_SOC_CLOCK_IN);
	if (ret < 0) {
		printk(KERN_ERR "can't set codec system clock\n");
		return ret;
	}

	ret = snd_soc_dai_set_sysclk(cpu_dai, OMAP_MCBSP_CLKR_SRC_CLKX, 0,
				SND_SOC_CLOCK_IN);
	if (ret < 0) {
		printk(KERN_ERR "can't set CPU system clock OMAP_MCBSP_CLKR_SRC_CLKX\n");
		return ret;
	}

	snd_soc_dai_set_sysclk(cpu_dai, OMAP_MCBSP_FSR_SRC_FSX, 0,
				SND_SOC_CLOCK_IN);
	if (ret < 0) {
		printk(KERN_ERR "can't set CPU system clock OMAP_MCBSP_FSR_SRC_FSX\n");
		return ret;
	}

	return 0;
}

static struct snd_soc_ops tam3517_ops = {
	.hw_params = tam3517_hw_params,
};


static const struct snd_soc_dapm_widget audio_widgets[] = {
        SND_SOC_DAPM_HP("Headphone Jack", NULL),
        SND_SOC_DAPM_LINE("Line In", NULL),
        SND_SOC_DAPM_LINE("Line Out", NULL),
        SND_SOC_DAPM_MIC("Mic Jack", NULL),
};

static const struct snd_soc_dapm_route audio_map[] = {
        { "Headphone Jack", NULL, "LHPOUT"},
        { "Headphone Jack", NULL, "RHPOUT"},

        { "Line Out", NULL, "LOUT" },
        { "Line Out", NULL, "ROUT" },

        { "LLINEIN", NULL, "Line In"},
        { "RLINEIN", NULL, "Line In"},

        { "MICIN", NULL, "Mic Jack"},
};

static int tam3517_aic23_init(struct snd_soc_codec *codec)
{
        snd_soc_dapm_new_controls(codec, audio_widgets,
                                  ARRAY_SIZE(audio_widgets));

        snd_soc_dapm_add_routes(codec, audio_map, ARRAY_SIZE(audio_map));

        snd_soc_dapm_enable_pin(codec, "Headphone Jack");
        snd_soc_dapm_enable_pin(codec, "Line In");
        snd_soc_dapm_enable_pin(codec, "Line Out");
        snd_soc_dapm_enable_pin(codec, "Mic Jack");

        snd_soc_dapm_sync(codec);

        return 0;
}

/* Digital audio interface glue - connects codec <--> CPU */
static struct snd_soc_dai_link tam3517_dai = {
	.name = "TLV320AIC23",
	.stream_name = "AIC23",
	.cpu_dai = &omap_mcbsp_dai[0],
	.codec_dai = &tlv320aic23_dai,
	.init = tam3517_aic23_init,
	.ops = &tam3517_ops,
};

/* Audio machine driver */
static struct snd_soc_card snd_soc_tam3517 = {
	.name = "tam3517",
	.platform = &omap_soc_platform,
	.dai_link = &tam3517_dai,
	.num_links = 1,
};

/* Audio subsystem */
static struct snd_soc_device tam3517_snd_devdata = {
	.card = &snd_soc_tam3517,
	.codec_dev = &soc_codec_dev_tlv320aic23,
};

static struct platform_device *tam3517_snd_device;

static int __init tam3517_soc_init(void)
{
	int ret;

	pr_info("TAM3517 EVM SoC init\n");

	tam3517_snd_device = platform_device_alloc("soc-audio", -1);
	if (!tam3517_snd_device) {
		printk(KERN_ERR "Platform device allocation failed\n");
		return -ENOMEM;
	}

	platform_set_drvdata(tam3517_snd_device, &tam3517_snd_devdata);
	tam3517_snd_devdata.dev = &tam3517_snd_device->dev;
	*(unsigned int *)tam3517_dai.cpu_dai->private_data = 0; /* McBSP1 */

	ret = platform_device_add(tam3517_snd_device);
	if (ret)
		goto err1;

	return 0;

err1:
	printk(KERN_ERR "Unable to add platform device\n");
	platform_device_put(tam3517_snd_device);

	return ret;
}

static void __exit tam3517_soc_exit(void)
{
	platform_device_unregister(tam3517_snd_device);
}

module_init(tam3517_soc_init);
module_exit(tam3517_soc_exit);

MODULE_AUTHOR("Hardy Weng <hardy.weng@technexion.com>");
MODULE_DESCRIPTION("ALSA SoC TAM3517 EVM");
MODULE_LICENSE("GPL v2");
